//
//  BTTestAppViewController.m
//  BTTestApp
//
//  Created by Y on 11/04/16.
//  Copyright 2011 京都産業大学. All rights reserved.
//

#import "BTTestAppViewController.h"

@implementation BTTestAppViewController



/*
// The designated initializer. Override to perform setup that is required before the view is loaded.
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}
*/

/*
// Implement loadView to create a view hierarchy programmatically, without using a nib.
- (void)loadView {
}
*/


/*
// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
    [super viewDidLoad];
}
*/


/*
// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}
*/

- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
	// Release any retained subviews of the main view.
	// e.g. self.myOutlet = nil;
}


- (void)dealloc {
    if (session != nil) {
		[session disconnectFromAllPeers];
		[session release];
		session = nil;
	}
	if (peerList != nil) {
		[peerList removeAllObjects];
		[peerList release];
		peerList = nil;
    }
	
	[super dealloc];
}


// ================================================

- (IBAction)msgClear:(id)sender {
	[msgView setText:@""];
}

// msgView にメッセージを書き込む、だけ
- (void)msgAdd: (NSString *)msg {
	if([msgView.text length] == 0) { 
		[msgView setText:msg];
	} else {
		NSString *appendedStr;
		appendedStr = [NSString stringWithFormat:@"%@\n%@", msgView.text, msg];
		[msgView setText:appendedStr];
		// 一番下の文字が見えるようにスクロール
		NSRange range = NSMakeRange(msgView.text.length-1, 1);
		[msgView scrollRangeToVisible:range];
	}
}

// 単に done を押したらキーボードをたたむだけ
- (IBAction)doneInputTextField: (id)sender {
    [sessionID resignFirstResponder];
    [displayName resignFirstResponder];
}



// 初期化処理
- (IBAction)initButton:(id)sender {

	// セッション初期化
	if (session != nil) {
		[session disconnectFromAllPeers];
		[session release];
		session = nil;
	}
	
	if ( [[sessionID text] length] <= 0 ) { // エラーメッセージを表示
		[self msgAdd:@"empty session ID"]; 
		return;
	}
	
	NSString *disp = nil; 
	if ( [[displayName text] length] > 0 ) { // 明示指定があれば入れ替え
		disp = [displayName text]; 
	}
	
	// GKsession を作り、ハンドラを設定し、広告をはじめる。これだけで準備は終わり。
	session = [[GKSession alloc] initWithSessionID: [sessionID text]
									   displayName: disp
									   sessionMode: GKSessionModePeer];
	session.delegate = self;  // いろいろ自分に定義
	[session setDataReceiveHandler:self withContext:nil]; // データ受信ハンドラを自分とする
	session.available = YES;  // 広告開始

	// ステータス表示に反映
	[self msgAdd:@"initialized."];
	NSLog(@"initialized.");
}

// 接続要求を受け付ける
- (IBAction)acceptButton:(id)sender {
    // 接続申請中リストを更新
    if ( peerList != nil ) {
        [peerList removeAllObjects];
        [peerList release];
        peerList = nil;
    }
	peerList = [[NSMutableArray alloc] initWithArray:[session peersWithConnectionState:GKPeerStateConnecting]];
    
    // 接続申請中のピアがいなければ処理終了
	if( [peerList count] <= 0 ) {
		[self msgAdd:@"No request to accept."];
		NSLog(@"No request to accept.");
		return;
	}
    
    // 接続申請に対して片っ端から許可
	NSString *peerStr = @"accepted";
    for ( NSString* aPeer in peerList) {
        [session acceptConnectionFromPeer: aPeer error: nil];
		peerStr = [peerStr stringByAppendingFormat:@" %@", [session displayNameForPeer: aPeer]];
    }

	[self msgAdd:peerStr];
	NSLog(@"accepted.");
}

// Connect
- (IBAction)connectButton:(id)sender {
    // この時点で接続可能なピアを捜す
    if ( peerList != nil ) {
        [peerList removeAllObjects];
        [peerList release];
        peerList = nil;
    }
	peerList = [[NSMutableArray alloc] initWithArray:[session peersWithConnectionState: GKPeerStateAvailable]];
    
    // 接続可能なピアがいなければ処理終了
	if( [peerList count] <= 0 ) {
		[self msgAdd:@"No peer to connect."];
		NSLog(@"No peer to connect.");
		return;
	}
    
    // 接続可能なピアに片っ端から接続申請
	NSString *peerStr = @"connecion requested to:";
    for ( NSString* aPeer in peerList) {
        [session connectToPeer: aPeer withTimeout: 1000];
		peerStr = [peerStr stringByAppendingFormat:@" %@", [session displayNameForPeer: aPeer]];
    }
	
	[self msgAdd:peerStr];
	NSLog(@"connecion requested.");
}

// Disconnect
- (IBAction)disconnectButton:(id)sender {
    // 接続中のリストを更新
    if ( peerList != nil ) {
        [peerList removeAllObjects];
        [peerList release];
        peerList = nil;
    }
	peerList = [[NSMutableArray alloc] initWithArray:[session peersWithConnectionState:GKPeerStateConnected]];
    
    // 接続中のピアがいなければ処理終了
	if( [peerList count] <= 0 ) {
		[self msgAdd:@"No connection."];
		NSLog(@"No connection.");
		return;
	}
    
    // 接続中のピアを一発切断
	[session disconnectFromAllPeers];
	session.available = YES; // 再び探索状態に
    
	[self msgAdd:@"disconnected."];
	NSLog(@"disconnected.");
}

// ピアの情報を集めて msgView に追記する（だけ）
- (void)getPeersWithState:(int)state withHeader:(NSString *)header {
	NSMutableArray *tempPeerList;
	tempPeerList = [[NSMutableArray alloc] initWithArray:[session peersWithConnectionState: state]];
	if( [tempPeerList count] <= 0 ) {
		[self msgAdd:[NSString stringWithFormat:@"no %@ peer", header]];
	} else {
		NSString *peerStr = [NSString stringWithFormat:@"%@ peers are:", header];
		for ( NSString* aPeer in tempPeerList) {
			peerStr = [peerStr stringByAppendingFormat:@" %@", [session displayNameForPeer: aPeer]];
		}
		[self msgAdd:peerStr];
	}
	
}

// 知っているピアを出す（どーでもいいので馬鹿っぽく三連ちゃん）
- (IBAction)showPeerButton:(id)sender {
    // 接続可能なピアを捜して表示
	[self getPeersWithState:GKPeerStateAvailable withHeader:@"available"];
    // 接続済みのピアを捜して表示
	[self getPeersWithState:GKPeerStateConnected withHeader:@"connected"];
    // 接続可能なピアを捜して表示
	[self getPeersWithState:GKPeerStateConnecting withHeader:@"con-requesting"];

	NSLog(@"show peers.");
}

// ===================================================== 
// データ送受信
// 受信メソッド名 receiveData は固定。setDataReceiveHandler: self によって GKsession の受信時ハンドラを自分とした。
// そしてこのとき呼ばれるメソッド名が receiveData で固定されている。（メソッドを受信時ハンドラとして登録していないのだな）

// データ受信
- (void)receiveData: (NSData *)data fromPeer: (NSString *)peer inSession: (GKSession *)l_session context: (void *)context {
    // 受信データの表示
	NSString* aStr = [[NSString alloc] initWithData: data encoding: NSASCIIStringEncoding];

	[self msgAdd:[NSString stringWithFormat:@"rcv #%@# from %@", aStr, [session displayNameForPeer: peer]]];
	NSLog(@"receive #%@#", aStr);
}

// Sendボタン タッチ時
-(IBAction)sendData: (id)sender{
    // 接続中配列の初期化
    if ( peerList != nil ) {
        [peerList removeAllObjects];
        [peerList release];
        peerList = nil;
    }
	peerList = [[NSMutableArray alloc] initWithArray:[session peersWithConnectionState:GKPeerStateConnected]];
    
	// 接続済みのピアがいなければ処理終了
	if( [peerList count] <= 0 ) {
		[self msgAdd:@"No peer connected."];
		NSLog(@"No peer connected.");
		return;
	}
	
    // メッセージの送信（全ての peer に送る）
	NSString *aStr = @"test-message";
	[session sendData: [aStr dataUsingEncoding: NSASCIIStringEncoding] 
              toPeers: peerList 
         withDataMode: GKSendDataReliable 
                error: nil];

    // ログを出す
	NSString *peerStr=@"";
    for ( NSString* aPeer in peerList) {
        [session acceptConnectionFromPeer: aPeer error: nil];
		peerStr = [peerStr stringByAppendingFormat:@" %@", [session displayNameForPeer: aPeer]];
    }
	
	[self msgAdd:[NSString stringWithFormat:@"snd #%@# to %@", aStr, peerStr]];
	NSLog(@"send #%@#", aStr);
}





// ===================================================== 
// GKsession の状態変更に対応するハンドラ。これ、ほぼお決まりというか何もすることがない。

// ステータス更新時の処理、といっても適当にメッセージを出すだけ
- (void)session: (GKSession *)l_session peer: (NSString *)peerID didChangeState: (GKPeerConnectionState)state{
	switch (state) {
        case GKPeerStateConnected: // 接続された 
			[self msgAdd:[NSString stringWithFormat:@"CHG: connected %@ (%@)",
						  peerID, [session displayNameForPeer: peerID]]];
			NSLog(@"Change status: GKPeerStateConnected.");	        
			break;
		case GKPeerStateDisconnected: // 接続が切れた
			// ここで [peerList removeObject:peerID]; とかしなくていいのか？
			[self msgAdd:[NSString stringWithFormat:@"CHG: disconnected %@ (%@)",
						  peerID, [session displayNameForPeer: peerID]]];
			 NSLog(@"Change status: GKPeerStateDisconnected.");	        
			break;
		case GKPeerStateAvailable: 
			 [self msgAdd:[NSString stringWithFormat:@"CHG: available %@ (%@)",
							peerID, [session displayNameForPeer: peerID]]];
			NSLog(@"Change status: GKPeerStateAvailable.");	        
			break;
        case GKPeerStateUnavailable: 
			  [self msgAdd:[NSString stringWithFormat:@"CHG: unavailable %@ (%@)",
							peerID, [session displayNameForPeer: peerID]]];
			NSLog(@"Change status: GKPeerStateDisconnected.");	        
			break;
        case GKPeerStateConnecting: 
			   [self msgAdd:[NSString stringWithFormat:@"CHG: con-requesting %@ (%@)",
							peerID, [session displayNameForPeer: peerID]]];
			NSLog(@"Change status: GKPeerStateConnecting.");	        
			break;
		default: 
			break;
    }
    
}

// 接続エラー時
- (void)session: (GKSession *) l_session connectionWithPeerFailed: (NSString *) peerID withError: (NSError *) error {
	[self msgAdd:[NSString stringWithFormat:@"connection failed %@ (%@) msg: %@",
				  peerID, [session displayNameForPeer: peerID], [error description]]];
	
	NSLog(@"connection failed, msg:%@", [error description]);
}

// セッションエラー時
- (void)session: (GKSession *) l_session didFailWithError: (NSError *) error {
	[self msgAdd:[NSString stringWithFormat:@"session error msg: %@",
				  [error description]]];
	
	NSLog(@"session error, msg:%@", [error description]);
}

// 接続要求受信時（この時はなにかダイアログでも出して積極的に接続させるなり勝手に接続させるなりする処理があり得る）
- (void)session: (GKSession *) l_session didReceiveConnectionRequestFromPeer: (NSString *) peerID {
	[self msgAdd:[NSString stringWithFormat:@"got con-request from %@ (%@)",
				  peerID, [session displayNameForPeer: peerID]]];
	
	NSLog(@"connection requested.");
}


@end

